# api_handler.py
import requests
import logging
from .config_handler import load_config

# VIGIL_API_BASE_URL = "http://127.0.0.1:8000/api/v1"
VIGIL_API_BASE_URL = "http://dockder-test-alb-2018306447.us-west-2.elb.amazonaws.com/api/v1"

# Set up logging
logging.basicConfig(level=logging.ERROR, filename='vijil.log', filemode='w',
                    format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

def get_headers():
    username, token = load_config()
    if not username or not token:
        raise ValueError("Username or token not found in configuration.")
    
    return {
        "username": username,
        "clitoken": token,
    }

def make_api_request(endpoint, method="get", params=None, data=None):
    url = f"{VIGIL_API_BASE_URL}/{endpoint}"
    headers = get_headers()
    
    try:
        if method.lower() == "get":
            response = requests.get(url, params=params, headers=headers)
        elif method.lower() == "post":
            response = requests.post(url, json=data, params=params, headers=headers)
        else:
            raise ValueError("Invalid HTTP method. Use 'get' or 'post'.")

        response.raise_for_status()
        return response.json()

    except Exception as e:
        logger.error(f"Error during API request: {e}")
        raise ValueError(f"An error occurred during the API request. Check the logs for details.")

def send_evaluation_request(model_type, model_name, probes, generations):
    endpoint = "evaluations/"
    data = {
        "model_type": model_type,
        "model_name": model_name,
        "probes": probes,
        "generations": generations,
    }
    return make_api_request(endpoint, method="post", data=data)

def job_status_request(id):
    endpoint = "evaluations/job_status"
    params = {"task_id": id}
    return make_api_request(endpoint, params=params)

def stop_job_request(id):
    endpoint = "evaluations/stop"
    params = {"task_id": id}
    return make_api_request(endpoint, method="post", params=params)

def stop_all_job_request():
    endpoint = "evaluations/stopall"
    return make_api_request(endpoint, method="post")

def delete_job_request(id):
    endpoint = "evaluations/remove"
    params = {"task_id": id}
    return make_api_request(endpoint, method="post", params=params)

def download_report_request(file_id):
    try:
        download_url = f"{VIGIL_API_BASE_URL}/evaluations/download_file?file_id={file_id}"
        response = requests.get(download_url)
        response.raise_for_status()

        filename = response.headers.get('Content-Disposition', '').split('filename=')[1].strip('"')

        with open(filename, 'wb') as file:
            file.write(response.content)

        return f"File downloaded successfully: {filename}"

    except requests.exceptions.RequestException as e:
        raise ValueError(f"Error during file download: {e}")

def list_job_request():
    endpoint = "jobs/"
    return make_api_request(endpoint)

def get_job_detail_request(id):
    endpoint = "jobs/detail"
    params = {"job_id": id}
    return make_api_request(endpoint, params=params)
